<?php

use Drupal\user\Entity\User;
use Drupal\cforge\EventSubscriber\MigrationSubscriber;
use Drupal\Core\Database\Database;
use Drupal\Core\StringTranslation\TranslatableMarkup;

 /**
  * Implements hook_migration_plugins_alter().
  *
  * Map all the permissions
  */
function _cforge_migration_plugins_alter(array &$definitions)  {
  //enable any optional modules from D7
  if ($database_info = \Drupal::state()->get('migrate_drupal_7')) {
    Database::addConnectionInfo('migrate', 'default', $database_info['database']);
    $enabled_d7_modules = Database::getConnection('default', 'upgrade')->select('system', 's')
      ->condition('status', 1)
      ->condition('type', 'module')
      ->fields('s', ['name'])
      ->execute()->fetchCol();
    sort($enabled_d7_modules);
    $map = [
      'cforge_solsearch' => 'smallads_index',
      'cforge_hosted' => 'cforge_ladmin'
    ];
    $module_handler = \Drupal::moduleHandler();
    foreach ($enabled_d7_modules as $module) {
      $new_module = isset($map[$module]) ? $map[$module] : $module;
      if (!$module_handler->moduleExists($new_module)) {
        $install_d8_modules[] = $new_module;
      }
    }
    if ($install_d8_modules) {
      $discovery = new Drupal\Core\Extension\ExtensionDiscovery(\Drupal::root());
      if ($modules = array_intersect($install_d8_modules, array_keys($discovery->scan('module')))) {
        drupal_set_message('Installing optional modules '.implode(', ', $modules) . ' from D7 need to refreshh the migration plugins', 'warning');
        \Drupal::service('module_installer')->install($modules);
      }
    }
  }
  else{
    trigger_error('Cannot find source Db', E_USER_ERROR);
  }

  $definitions['d7_block']['process']['region']['map']['sky_seldulac']['sky_seldulac'] = [
    'help' => 'top',
    'headerright' => null,
    'navigation' => 'top',
    'contentbottom' => 'content',
    'contentfooter' => 'footer',
    'footer' => null
  ];
  $definitions['d7_filter_format']['process']['filters']['id']['map'] = MigrationSubscriber::FILTER_FORMATS;
  $definitions['d7_filter_format']['process']['filters']['id']['default_value'] = 'plain_text';

  //in 8.4-rc2 menu items are failing validation because their routes haven't been migrated yet
  $definitions['d7_menu_links']['migration_dependencies']['required'][] = 'd7_node';
  $definitions['d7_menu_links']['migration_dependencies']['required'][] = 'd7_contact';
  // Simplify the comment  type process, which is confused by the renamings of things.
  // Then see
  $definitions['d7_comment']['process']['comment_type'] = 'comment_type';
  foreach ($definitions as &$def) {
    if (isset($def['process']['event_date'])) {
      $def['process']['date'] = $def['process']['event_date'];
      $def['process']['date']['process']['value']['to_format'] = 'Y-m-d\TH:i:s';
      unset($def['process']['event_date']);
    }
  }
  $definitions['d7_user']['process']['portrait']  = $definitions['d7_user']['process']['user_picture'];
  unset($definitions['d7_user']['process']['user_picture']);

  $definitions['d7_file']['source']['constants']['source_base_path'] = DRUPAL_ROOT;
  $definitions['d7_menu']['process']['id']['bypass'] = 'FALSE';//don't import any unknown menus

  unset(
    // I don't know why these are being included
    $definitions['user_profile_field'],
    $definitions['user_profile_field_instance'],
    $definitions['user_profile_entity_display'],
    $definitions['user_profile_entity_form_display'],
    // Do not inherit any view modes
    $definitions['d7_view_mode'],
    $definitions['d7_field_instance_widget_settings'],
    $definitions['d7_comment_entity_form_display'],
    $definitions['d7_comment_entity_display'],
    $definitions['d7_comment_field_instance'],
    $definitions['d7_filter_format']['process']['filters']['id']['bypass']
  );
}

/**
 * Implements hook_migrate_prepare_row().
 *
 * Do it here for all the features as well
 *
 * @return NULL|boolean
 *
 * @see migrate.api.php
 */
function _cforge_migrate_prepare_row($row, $source, $migration) {
  switch ($source->getPluginId()) {
    case 'd7_field':
    case 'd7_field_instance':
      //Ensure that none of the d8 fields are overwritten by obselete or renamed d7 fields
      if (in_array($source->getPluginId(), ['d7_field', 'd7_field_instance'])) {
        $existing_fields = [
          'account_offline',
          'body',
          'cforge_docs_categories',
          'comment_body',
          'event_date',
          'galleries',
          'image',
          'manage_notes',
          'manage_responsibility',//removed
          'offers_wants_categories',
          'offers_wants_types',
          'profile_phones',
          'profile_address',
          'title', // Prevents 'base_field_override' fields being created
          'upload',//became 'attached'
          'upload_private', // Became 'attached',
        ];
        $field_name = $row->getSourceProperty('field_name');
        if (in_array($field_name, $existing_fields)) {
          return FALSE;
        }
      }
      break;

    case 'd7_filter_format':
      $ignore = array_keys(MigrationSubscriber::FILTER_FORMATS);
      if (in_array($row->getSourceProperty('format'), $ignore)) {
        return FALSE;
      }
      break;

    case 'd7_node_type':
      // don't migrate document
      if (in_array($row->getSourceProperty('type'), ['document', 'community_task', 'event'])) {
        return FALSE;
      }
      break;

    case 'd7_comment':
      // Skip comments where the user has been deleted.
      $uid = $row->getSourceProperty('uid');
      return (bool)User::load($uid);

    case 'menu_link':
      // Only migrate menu links which are not already in the menu tree
      //use the title to identify menu links
      $title = $row->getSourceProperty('link_title');
      $markup = new TranslatableMarkup($title);
      $query = \Drupal::database()->select('menu_tree', 'mt')
        ->fields('mt');
      return !$query->condition($query->orConditionGroup()
        ->condition('title', serialize($markup))
        ->condition('title', serialize($title))
      )->execute()->fetch();

    case 'd7_theme_settings':
      if (in_array($row->getSourceProperty('name'), ['theme_sky_settings', 'theme_garland_settings']))  {
        return FALSE;
      }
      break;
    case 'contact_category':
      // This only works in English so far
      if (strlen($row->getSourceProperty('category')) > 32) {
        return FALSE;
      }
      break;

    case 'block':
      $ignoreDeltas = ['visitors', 'secondary-menu', 'form', 'user_online'];// the last one is the search form
      $ignoreThemes = ['bartik', 'seven'];
      if (in_array($row->getSourceProperty('delta'), $ignoreDeltas) or in_array($row->getSourceProperty('theme'), $ignoreThemes)) {
        return FALSE;
      }
      echo "\nAllowing block ".$row->getSourceProperty('delta');// why is user_online getting through?
      break;

    case 'embedded_data'://block_content_body_field
      if ($migration->id() == 'block_content_type')return FALSE;
      if ($migration->id() == 'block_content_entity_display')return FALSE;
      if ($migration->id() == 'block_content_entity_form_display')return FALSE;

      if ($migration->id() == 'block_content_entity_form_display')return TRUE;
      echo $migration->id(); print_r($row->getSource());
      return FALSE;

    case 'menu':
      if ($row->getSourceProperty('menu_name') == 'devel') {
        return FALSE;
      }

    case 'd7_comment_type':
    case 'd7_node_title_label':
      return FALSE;

    case 'd7_comment_field':
      echo "\n_cforge_migrate_prepare_row skipping row with sourcePluginId d7_comment_field";
      return FALSE;

  }

}
